import iris
import numpy as np
import sys
from configparser import SafeConfigParser
import pickle
import glob

def nearest_ind(data,val):
    '''Returns the index of the data array which has the value closest to val'''
    ind = abs(data-val).argmin() 
    return ind
    

def load_cube(f,name,ndim=3):
    '''Remove halo of repeating values at east and west boundaries if it exists'''
    c = iris.load(f,name)[0]
    if c.ndim==ndim+1:
        c=c[0]
    if c.ndim==3:
        c=c[:,1:-1,1:-1]
    elif c.ndim==2:
        c=c[1:-1,1:-1]
    return c
   
def load_ini(ini_file):
    '''Load definitions and set up'''
    parser = SafeConfigParser()
    parser.read(ini_file) 
    grid_file = parser.get('inputs','grid_file')
    mask_file = parser.get('inputs','mask_file')
    names={}
    for k in parser.options('names'):
        names[k] = parser.get('names',k)
    slat = float(parser.get('region','south_lat'))
    nlat = float(parser.get('region','north_lat'))
    dlat = float(parser.get('region','lat_spacing'))
    i0 = float(parser.get('region','lati'))
    lats = np.arange(slat,nlat,dlat)
    region = {'nlat':nlat,'slat':slat,'lats':lats,'i0':i0}
    return names,grid_file,mask_file,region

def get_grid(names,grid_file,mask_file,j):
    '''Get grid information for j coordinate'''
    #Settings
    dx = load_cube(grid_file,names['dx'])[:,j,:]
    dz = load_cube(grid_file,names['dz'])[:,j,:]
    #Get mask
    tmp = load_cube(mask_file,names['t'])[:,j,:]
    maskatl = tmp.data.mask
    dx_masked = np.ma.masked_array(dx.data,mask=maskatl[0][None,:])
    dz_masked = np.ma.masked_array(dz.data,mask=maskatl)
    area = (dx_masked*dz_masked).sum()
    L = (dx_masked*dz_masked).sum(axis=1)
    grid = {'dx':dx_masked,'dz':dz_masked,'A':area,'L':L,'lat':tmp.coord(names['lat']).points,'z':tmp.coord(names['z']).points,'j':j}
    return grid

FW_FACTOR=-1.0/35.0/1.0e6 # convert from PSU m3/s to Sv of FW

def calc_fov(V,S,VS,grid,j,output_full=False):
    '''
    Split transport into components 
    
       V = vbtp + vav(z) + vanom(x,z) 
       S =        sav(z) + sanom(x,z)
       
    where
     
       vbtp = vtot/A 
       vav = vint/L
       vtot = sum_xz(V dx dz)
       vint = sum_x((V-vbtp) dx dz)
       A = sum_xz(dx dz)
       L = sum_x(dx dz)
       
    Then calculate
    
       ftot = sum_xz(V S dx dz)*FW_FACTOR
       fbtp = vtot*smean*FW_FACTOR
       fov = sum_z(vint sav )*FW_FACTOR
       faz = sum_xz(vanom sanom dx dz)*FW_FACTOR
       
    inputs:
       V (2d masked array) - velocity data
       S (2d masked array) - salinity data
       grid (dic) - dictionary containing output from load_grid
       
    output:
       tuple of ftot,fbtp,fov,faz
    '''
    dx=grid['dx']
    dz=grid['dz']
    vtot = (V*dx*dz).sum()
    smean = (S*dx*dz).sum()/grid['A']
    fbtp = vtot*smean*FW_FACTOR
    vbcl = V - vtot/grid['A']
    vint = (vbcl*dx*dz).sum(axis=1)
    sav = (S*dx*dz).sum(axis=1)/grid['L']
    vanom = vbcl - (vint/grid['L'])[:,None]
    sanom = S - sav[:,None]
    ftot = (V*S*dx*dz).sum()*FW_FACTOR
    fov = (vint*sav).sum()*FW_FACTOR
    faz = (vanom*sanom*dx*dz).sum()*FW_FACTOR
    ffull = (VS*dx*dz).sum()*FW_FACTOR
    if output_full:
       output = ftot,fbtp,fov,faz,vtot,smean,ffull
    else:
       output = ftot,fbtp,fov,faz,ffull
    return output


def loop_fov(vf,sf,names,grid_file,mask_file,region):
    '''Read in data and loop through meridional coordinate'''
    s = load_cube(sf,names['s'])
    v = load_cube(vf,names['v'])
    vs = load_cube(vf,names['vs'])
    av_lat = v.coord('latitude').points[:,int(region['i0'])]
    ftot_all = []
    fbtp_all = []
    fov_all = []
    faz_all = [] 
    ffull_all = []    
    vint_all = []
    sav_all = []
    for l in region['lats']:
       print(l)
       j=nearest_ind(av_lat,l)
       grid = get_grid(names,grid_file,mask_file,j)
       ftot,fbtp,fov,faz,vint,sav,ffull = calc_fov(v[:,j,:].data,s[:,j,:].data,vs[:,j,:].data,grid,j,output_full=True)
       ftot_all.append(ftot)
       fbtp_all.append(fbtp)
       fov_all.append(fov)
       faz_all.append(faz)
       ffull_all.append(ffull)
       vint_all.append(vint)
       sav_all.append(sav)
    dic = {'ftot':np.array(ftot_all),'fbtp':np.array(fbtp_all),'fov':np.array(fov_all),'faz':np.array(faz_all),'ffull':np.array(ffull_all),'lat':region['lats'],'vint':np.array(vint_all),'sav':np.array(sav_all)}
    return dic

if __name__ == '__main__':
    '''
    Usage: python calc_fw_trans.py vfile.nc sfile.nc fileout.pkl inifile
    
    To be used with parallel computing script by looping through monthly V and S data
    '''
    vf,sf,fout,ifile = sys.argv[1:]
    print(vf,sf,fout)

    ini_file=ifile
    names,grid_file,mask_file,region = load_ini(ini_file)

    dic=loop_fov(vf,sf,names,grid_file,mask_file,region)

    output=open(fout,'wb')
    pickle.dump(dic,output)
    output.close()
